
/**********************************************************************
 *           gamefield.cpp  -  description
 *           Project: "Hexatrolic"
 *
 *           (C) 2002 by Helge Plehn
 *           29.11.2002
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
**********************************************************************/

#include "gamefield.h"

#include <qpe/resource.h>

#include <qpainter.h>
#include <qpixmap.h>

#include <math.h>
#include <qmessagebox.h>
//#include <qsound.h>

#include <qcursor.h>
#include <qpoint.h>
#include <qevent.h>
#include <float.h>

#include "qembed_images.h"



// Just call the generated function
// "name" is the original image filename WITHOUT the extension
QImage myFindImage(const char* name)
{
    return qembed_findImage(name);
}

 /*

// Just call the generated function
// "name" is the original filename WITH the extension
QByteArray myFindData(const char* name)
{
    return qembed_findData(name);
}

*/


GameField::GameField( QWidget *parent, const char *name )
: QWidget( parent, name )
{
 GameMode = 0;
 AnzPlayer = 1;
 BallPosX_old=1000;
 BallPosY_old=1000;
 BallMode = 0;

 PadMoveTo = QPoint(0,0);
 PadSpeed = QPoint(0,0);

 balls = new QLCDNumber(1 , this, "balls"  );
 balls->setGeometry(0, 175, 21, 15 );
 balls->setBackgroundColor( QColor(0, 0, 0) );
 balls->setFrameStyle ( 0 );
 balls->display( 5 );

 level = new QLCDNumber(4 , this, "level"  );
 level->setGeometry(200, 175, 40, 15 );
 level->setBackgroundColor( QColor(0, 0, 0) );
 level->setFrameStyle ( 0 );
 level->display( 1 );


 for( int i = 0 ; i < 3 ; i++ )
 {
 	PadPosX_old[i]=1000;
 	PadPosY_old[i]=1000;
 	PadMode[i] = 0;
 };

  ServeBall = false;


 //loadbitmaps

// BitBlt will be faster
//QPixmap::setDefaultOptimization ( QPixmap::BestOptim ) ;

 bool ok = true;


//QBitmap & QBitmap::operator= ( const QImage & image )

// if ( ok ) ok = hexagonimage.load("/usr/games/hexatrolic/bitmaps/hexagon.png",0);
// if ( ok ) ok = padimage.load("/usr/games/hexatrolic/bitmaps/pad.png",0);

// if ( ok ) ok = hexagonimage.load("$QPEDIR/pis/hexagon.png",0);
// if ( ok ) ok = padimage.load("pad.png",0);
 hexagonimage = Resource::loadPixmap("hexatrolic/hexatrolic_hexagon");
 if ( hexagonimage.isNull() )
 {
   //Image could not be loaded
   QMessageBox::critical ( 0, tr("Error!"), tr("Couldn't access Bitmaps! \n"
                                               "hexatrolic_hexagon.png \n"
                                               "not found in \n"
                                               "$OPIEDIR/pics/hexatrolic/") );
    close();
 }

 padimage = Resource::loadPixmap("hexatrolic/hexatrolic_pads");
 if ( padimage.isNull() )
 {
   //Image could not be loaded
   QMessageBox::critical ( 0, tr("Error!"), tr("Couldn't access Bitmaps! \n"
                                               "hexatrolic_pads.png \n"
                                               "not found in \n"
                                               "$OPIEDIR/pics/hexatrolic") );
    close();
 }



 if ( ok ) ok = ballimage.convertFromImage(myFindImage("ball"),0);
 if ( ok ) ok = ballmask.convertFromImage(myFindImage("ballmask"),0);
 if ( ok ) ok = padmask.convertFromImage(myFindImage("padmask"),0);
 if ( ok ) ok = stoneimage.convertFromImage(myFindImage("stone"));
 if ( ok ) ok = stonemask.convertFromImage(myFindImage("stonemask"));
 if ( ok ) ok = tor1image.convertFromImage(myFindImage("tor1"));
 if ( ok ) ok = tor1mask.convertFromImage(myFindImage("tor1mask"));
 if ( ok ) ok = tor2image.convertFromImage(myFindImage("tor2"));
 if ( ok ) ok = tor2mask.convertFromImage(myFindImage("tor2mask"));
 if ( ok ) ok = tor3image.convertFromImage(myFindImage("tor3"));
 if ( ok ) ok = tor3mask.convertFromImage(myFindImage("tor3mask"));
 if ( ok ) ok = tor4image.convertFromImage(myFindImage("tor4"));
 if ( ok ) ok = tor4mask.convertFromImage(myFindImage("tor4mask"));

 //ballmask = ballmask.operator= ( myFindImage("ballmask") );	// geht auch!

 //if ( ok ) ok = ballmask.load("/usr/games/hexatrolic/bitmaps/ballmask.bmp",0);
 //if ( ok ) ok = padimage.convertFromImage(myFindImage("pad"),0);
 //if ( ok ) ok = padmask.load("/usr/games/hexatrolic/bitmaps/padmask.bmp",0);
 // if ( ok ) ok = tor1mask.load("/usr/games/hexatrolic/bitmaps/tor1mask.bmp",0);
 // if ( ok ) ok = tor2mask.load("/usr/games/hexatrolic/bitmaps/tor2mask.bmp",0);
 // if ( ok ) ok = tor3image.load("/usr/games/hexatrolic/bitmaps/tor3s.bmp",0);
 // if ( ok ) ok = tor3mask.load("/usr/games/hexatrolic/bitmaps/tor3smask.bmp",0);
 //if ( ok ) ok = tor4image.load("/usr/games/hexatrolic/bitmaps/tor4s.bmp",0);
 //if ( ok ) ok = tor4mask.load("/usr/games/hexatrolic/bitmaps/tor4smask.bmp",0);


 if ( !ok )
 {
   //Image could not be loaded
	  QMessageBox::critical ( 0, tr("Error!"), tr("Couldn't access Bitmaps!" ));
    close();
 }

 // Optimize Speed for Bitblt
 gameimage.setOptimization ( QPixmap::BestOptim ) ;
 ballcopy.setOptimization ( QPixmap::BestOptim ) ;
 padcopy.setOptimization ( QPixmap::BestOptim ) ;
 stonecopy.setOptimization ( QPixmap::BestOptim ) ;
 ballimage.setOptimization ( QPixmap::BestOptim ) ;
 padimage.setOptimization ( QPixmap::BestOptim ) ;
 tor1image.setOptimization ( QPixmap::BestOptim ) ;
 tor2image.setOptimization ( QPixmap::BestOptim ) ;
 tor3image.setOptimization ( QPixmap::BestOptim ) ;
 tor4image.setOptimization ( QPixmap::BestOptim ) ;

 gameimage = hexagonimage;

 ballcopy = QPixmap( 8, 8, -1 );   //Bits per Pixel = -1 sets the current Bit Depth of the system
 padcopy = QPixmap( 72, 24, -1 );
 stonecopy = QPixmap( 48, 43, -1 );

 bitBlt( &stonecopy, 0, 0, &gameimage, 96, 84, 48, 43, CopyROP , true);

 //Set the masks for the sprites
 ballimage.setMask( ballmask );
 padimage.setMask( padmask );
 stoneimage.setMask( stonemask );
 tor1image.setMask( tor1mask );
 tor2image.setMask( tor2mask );
 tor3image.setMask( tor3mask );
 tor4image.setMask( tor4mask );

 setBackgroundPixmap( gameimage );      // set desktop pixmap



 // Defines all the Data-Arrays
 DefineArrays();

 setMouseTracking ( true );	// Enables MouseMoveEvents without any Mousebutton pressed

}


void GameField::SetPlayer( int Anzahl )
{
 AnzPlayer = Anzahl;
 if(AnzPlayer == 1 )
 {
  gameimage = hexagonimage;
  setBackgroundPixmap( gameimage );               // set desktop pixmap
 };

 if(AnzPlayer == 2 )
 {
  gameimage = hexagonimage;
  //bitBlt( &gameimage, 80, 0, &tor2back, 0, 0, 80, 11, CopyROP , true );
  bitBlt( &gameimage, 80, 0, &tor2image, 0, 0, 80, 11, CopyROP , true );
  setBackgroundPixmap( gameimage );               // set desktop pixmap
 };

 if(AnzPlayer == 3 )
 {
  gameimage = hexagonimage;
   SetLeftGoal ( true );
   SetRightGoal ( true );

//  bitBlt( &gameimage, 13, 20, &tor3image, 0, 0, 48, 72, CopyROP , true );
//  bitBlt( &gameimage, 179, 20, &tor4image, 0, 0, 48, 72, CopyROP , true );
  setBackgroundPixmap( gameimage );               // set desktop pixmap
 };
}

void GameField::SetLeftGoal( bool SimpleCopy )
{
// Setzt das linke, obere Tor. Entweder nur Kopieren (SimpleCopy==true) oder
// mit Maske (SimpleCopy==false) und berschreiben des Bereiches hinter dem Tor
 for( int i = 0 ; i < 6; i++ )
 {
   bitBlt( &gameimage, 44 - 3*i, 20 + 6*i , &tor3image, 0, 6*i, 16, 6, CopyROP , SimpleCopy );
   bitBlt( &gameimage, 23 - 3*i, 56 + 6*i , &tor3image, 0, 36 + 6*i, 16, 6, CopyROP , SimpleCopy );
   if ( !SimpleCopy )
	{
	  bitBlt( &gameimage, 0 , 20 + 6*i , &hexagonimage , 0 , 0,  44- 3*i, 6, CopyROP , true );
	  bitBlt( &gameimage, 0 , 56 + 6*i , &hexagonimage , 0 ,  0,  23 - 3*i, 6, CopyROP , true );
	};
 };
}

void GameField::SetRightGoal( bool SimpleCopy )
{
// Setzt das linke, obere Tor. Entweder nur Kopieren (SimpleCopy==true) oder
// mit Maske (SimpleCopy==false) und berschreiben des Bereiches hinter dem Tor
 for( int i = 0 ; i < 6; i++ )
 {
   bitBlt( &gameimage, 180 + 3*i, 20 + 6*i , &tor4image, 0, 6*i, 16, 6, CopyROP , SimpleCopy );
   bitBlt( &gameimage, 201 + 3*i, 56 + 6*i , &tor4image, 0, 36 + 6*i, 16, 6, CopyROP , SimpleCopy );
   if ( !SimpleCopy )
	{
	  bitBlt( &gameimage, 196 + 3*i , 20 + 6*i , &hexagonimage , 0 , 0,  44- 3*i, 6, CopyROP , true );
	  bitBlt( &gameimage, 217 + 3*i , 56 + 6*i , &hexagonimage , 0 ,  0,  23 - 3*i, 6, CopyROP , true );
	};
 };
}


void GameField::DefineArrays()
{
//Array-Werte zuweisen

    //X-Position der Steine (0-53)
    const int InitSteinXpos[54] = {104, 108, 112, 116, 120, 124, 128, 100, 104, 108, 112, 116, 120, 124, 128, 132, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 100, 104, 108, 112, 116, 120, 124, 128, 132, 104, 108, 112, 116, 120, 124, 128};

    //Y-Position der Steine (0-53)
    const int InitSteinYpos[54] = {84, 84, 84, 84, 84, 84, 84, 91, 91, 91, 91, 91, 91, 91, 91, 91, 98, 98, 98, 98, 98, 98, 98, 98, 98, 98, 98, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 112, 112, 112, 112, 112, 112, 112, 112, 112, 119, 119, 119, 119, 119, 119, 119};

    //Ausrichtung der Steine (0-53)
    const bool InitSteinOrientierung[54] = {1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0};

    //Minimale X-Position fr Kollision mit Stein (0-53)
    const float InitSteinMinX[54] = {97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 89.40192379, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 89.40192379, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238};

    //Maximale X-Position fr Kollision mit Stein (0-53)
    const float InitSteinMaxX[54] = {110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 142.5980762, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 142.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762};

    //Minimale Y-Position fr Kollision mit Stein (0-53)
    const float InitSteinMinY[54] = {78.5, 77, 78.5, 77, 78.5, 77, 78.5, 85.5, 84, 85.5, 84, 85.5, 84, 85.5, 84, 85.5, 92.5, 91, 92.5, 91, 92.5, 91, 92.5, 91, 92.5, 91, 92.5, 98, 99.5, 98, 99.5, 98, 99.5, 98, 99.5, 98, 99.5, 98, 105, 106.5, 105, 106.5, 105, 106.5, 105, 106.5, 105, 112, 113.5, 112, 113.5, 112, 113.5, 112};

    //Maximale Y-Position fr Kollision mit Stein (0-53)
    const float InitSteinMaxY[54] = {90, 88.5, 90, 88.5, 90, 88.5, 90, 97, 95.5, 97, 95.5, 97, 95.5, 97, 95.5, 97, 104, 102.5, 104, 102.5, 104, 102.5, 104, 102.5, 104, 102.5, 104, 109.5, 111, 109.5, 111, 109.5, 111, 109.5, 111, 109.5, 111, 109.5, 116.5, 118, 116.5, 118, 116.5, 118, 116.5, 118, 116.5, 123.5, 125, 123.5, 125, 123.5, 125, 123.5};

    //Steigung der den Stein begrenzenden Graden (0-5)
    const float InitSteinA[6] = {0, 1.75, -1.732050808, 0, 1.732050808, -1.75};

    //Y-Achsenabschnitt der den Stein begrenzenden Graden(0-5)(0-53)
    const float InitSteinB[6][54] = {{78.5, 88.5, 78.5, 88.5, 78.5, 88.5, 78.5, 85.5, 95.5, 85.5, 95.5, 85.5, 95.5, 85.5, 95.5, 85.5, 92.5, 102.5, 92.5, 102.5, 92.5, 102.5, 92.5, 102.5, 92.5, 102.5, 92.5, 109.5, 99.5, 109.5, 99.5, 109.5, 99.5, 109.5, 99.5, 109.5, 99.5, 109.5, 116.5, 106.5, 116.5, 106.5, 116.5, 106.5, 116.5, 106.5, 116.5, 123.5, 113.5, 123.5, 113.5, 123.5, 113.5, 123.5},
    {-108.0466334, -95.95336663, -122.0466334, -109.9533666, -136.0466334, -123.9533666, -150.0466334, -94.04663337, -81.95336663, -108.0466334, -95.95336663, -122.0466334, -109.9533666, -136.0466334, -123.9533666, -150.0466334, -80.04663337, -67.95336663, -94.04663337, -81.95336663, -108.0466334, -95.95336663, -122.0466334, -109.9533666, -136.0466334, -123.9533666, -150.0466334, -53.95336663, -80.04663337, -67.95336663, -94.04663337, -81.95336663, -108.0466334, -95.95336663, -122.0466334, -109.9533666, -136.0466334, -123.9533666, -53.95336663, -80.04663337, -67.95336663, -94.04663337, -81.95336663, -108.0466334, -95.95336663, -122.0466334, -109.9533666, -53.95336663, -80.04663337, -67.95336663, -94.04663337, -81.95336663, -108.0466334, -95.95336663},
    {277.0614872, 257.133284, 290.9178937, 270.9896904, 304.7743001, 284.8460969, 318.6307066, 277.133284, 257.2050808, 290.9896904, 271.0614872, 304.8460969, 284.9178937, 318.7025034, 298.7743001, 332.5589098, 277.2050808, 257.2768775, 291.0614872, 271.133284, 304.9178937, 284.9896904, 318.7743001, 298.8460969, 332.6307066, 312.7025034, 346.4871131, 257.3486743, 291.133284, 271.2050808, 304.9896904, 285.0614872, 318.8460969, 298.9178937, 332.7025034, 312.7743001, 346.5589098, 326.6307066, 271.2768775, 305.0614872, 285.133284, 318.9178937, 298.9896904, 332.7743001, 312.8460969, 346.6307066, 326.7025034, 285.2050808, 318.9896904, 299.0614872, 332.8460969, 312.9178937, 346.7025034, 326.7743001},
    {90, 77, 90, 77, 90, 77, 90, 97, 84, 97, 84, 97, 84, 97, 84, 97, 104, 91, 104, 91, 104, 91, 104, 91, 104, 91, 104, 98, 111, 98, 111, 98, 111, 98, 111, 98, 111, 98, 105, 118, 105, 118, 105, 118, 105, 118, 105, 112, 125, 112, 125, 112, 125, 112},
    {-83.20508076, -116.9896904, -97.06148722, -130.8460969, -110.9178937, -144.7025034, -124.7743001, -69.27687753, -103.0614872, -83.13328399, -116.9178937, -96.98969045, -130.7743001, -110.8460969, -144.6307066, -124.7025034, -55.3486743, -89.13328399, -69.20508076, -102.9896904, -83.06148722, -116.8460969, -96.91789368, -130.7025034, -110.7743001, -144.5589098, -124.6307066, -75.20508076, -55.27687753, -89.06148722, -69.13328399, -102.9178937, -82.98969045, -116.7743001, -96.84609691, -130.6307066, -110.7025034, -144.4871131, -75.13328399, -55.20508076, -88.98969045, -69.06148722, -102.8460969, -82.91789368, -116.7025034, -96.77430014, -130.5589098, -75.06148722, -55.13328399, -88.91789368, -68.98969045, -102.7743001, -82.84609691, -116.6307066},
    {255.9533666, 282.0466334, 269.9533666, 296.0466334, 283.9533666, 310.0466334, 297.9533666, 255.9533666, 282.0466334, 269.9533666, 296.0466334, 283.9533666, 310.0466334, 297.9533666, 324.0466334, 311.9533666, 255.9533666, 282.0466334, 269.9533666, 296.0466334, 283.9533666, 310.0466334, 297.9533666, 324.0466334, 311.9533666, 338.0466334, 325.9533666, 282.0466334, 269.9533666, 296.0466334, 283.9533666, 310.0466334, 297.9533666, 324.0466334, 311.9533666, 338.0466334, 325.9533666, 352.0466334, 296.0466334, 283.9533666, 310.0466334, 297.9533666, 324.0466334, 311.9533666, 338.0466334, 325.9533666, 352.0466334, 310.0466334, 297.9533666, 324.0466334, 311.9533666, 338.0466334, 325.9533666, 352.0466334}};

    //minimaler X-Wert fr Kollision mit Grade von Stein (0-5)(0-53)
    const float InitSteinXmin[6][54] = {{101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 133.4019238, 93.40192379, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 133.4019238, 97.40192379, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238, 129.4019238, 101.4019238, 105.4019238, 109.4019238, 113.4019238, 117.4019238, 121.4019238, 125.4019238},
    {106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 98.59807621, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 89.40192379, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238},
    {108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132, 104, 97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238, 136, 100, 93.40192379, 108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132, 125.4019238, 140, 89.40192379, 104, 97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238, 136, 129.4019238, 93.40192379, 108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132, 125.4019238, 97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238},
    {100, 104, 108, 112, 116, 120, 124, 96, 100, 104, 108, 112, 116, 120, 124, 128, 92, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 92, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 96, 100, 104, 108, 112, 116, 120, 124, 128, 100, 104, 108, 112, 116, 120, 124},
    {97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238, 93.40192379, 108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132, 125.4019238, 89.40192379, 104, 97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238, 136, 129.4019238, 100, 93.40192379, 108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132, 125.4019238, 140, 104, 97.40192379, 112, 105.4019238, 120, 113.4019238, 128, 121.4019238, 136, 108, 101.4019238, 116, 109.4019238, 124, 117.4019238, 132},
    {97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 89.40192379, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 98.59807621, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762}};

    //maximaler X-Wert fr Kollision mit Grade von Stein (0-5)(0-53)
    const float InitSteinXmax[6][54] = {{106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 98.59807621, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 98.59807621, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 138.5980762, 102.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762, 134.5980762, 106.5980762, 110.5980762, 114.5980762, 118.5980762, 122.5980762, 126.5980762, 130.5980762},
    {110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 142.5980762, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 133.4019238, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238},
    {110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762, 106.5980762, 100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124, 138.5980762, 102.5980762, 96, 110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762, 128, 142.5980762, 92, 106.5980762, 100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124, 138.5980762, 132, 96, 110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762, 128, 100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124},
    {108, 112, 116, 120, 124, 128, 132, 104, 108, 112, 116, 120, 124, 128, 132, 136, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 104, 108, 112, 116, 120, 124, 128, 132, 136, 108, 112, 116, 120, 124, 128, 132},
    {100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124, 96, 110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762, 128, 92, 106.5980762, 100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124, 138.5980762, 132, 102.5980762, 96, 110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762, 128, 142.5980762, 106.5980762, 100, 114.5980762, 108, 122.5980762, 116, 130.5980762, 124, 138.5980762, 110.5980762, 104, 118.5980762, 112, 126.5980762, 120, 134.5980762},
    {101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 93.40192379, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 133.4019238, 102.5980762, 97.40192379, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762, 129.4019238, 142.5980762, 106.5980762, 101.4019238, 114.5980762, 109.4019238, 122.5980762, 117.4019238, 130.5980762, 125.4019238, 138.5980762, 110.5980762, 105.4019238, 118.5980762, 113.4019238, 126.5980762, 121.4019238, 134.5980762}};

    //Steinfarben fr jeden Level (0-11)(0-53)
    const int InitSteinFarbenLevel[12][54] = {{6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6, 5, 6, 6, 6},
    {3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 3, 3, 3, 3, 0, 0, 4, 4, 4, 0, 0, 3, 3, 3, 3, 0, 0, 4, 4, 4, 0, 0, 3, 3, 3, 3, 0, 0, 0, 0, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3},
    {1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 2, 0, 0, 1, 0, 1, 1, 0, 2, 2, 2, 2, 2, 0, 1, 1, 1, 1, 0, 2, 2, 2, 2, 2, 0, 1, 1, 0, 1, 0, 0, 2, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {4, 4, 0, 0, 0, 4, 4, 0, 0, 4, 4, 0, 4, 4, 0, 0, 4, 0, 4, 0, 4, 4, 4, 0, 4, 0, 4, 4, 0, 4, 0, 4, 4, 4, 0, 4, 0, 4, 0, 0, 4, 4, 0, 4, 4, 0, 0, 4, 4, 0, 0, 0, 4, 4},
    {0, 0, 2, 2, 2, 0, 0, 2, 2, 2, 2, 0, 2, 2, 2, 2, 0, 2, 2, 0, 1, 1, 1, 0, 2, 2, 0, 0, 2, 2, 0, 1, 1, 1, 0, 2, 2, 0, 2, 2, 2, 2, 0, 2, 2, 2, 2, 0, 0, 2, 2, 2, 0, 0},
    {4, 4, 4, 0, 4, 4, 4, 0, 4, 0, 0, 0, 0, 0, 4, 0, 4, 4, 0, 0, 0, 0, 0, 0, 0, 4, 4, 4, 4, 0, 0, 0, 0, 0, 0, 0, 4, 4, 0, 4, 0, 0, 0, 0, 0, 4, 0, 4, 4, 4, 0, 4, 4, 4},
    {6, 6, 0, 6, 0, 6, 6, 6, 0, 0, 0, 3, 0, 0, 0, 6, 6, 0, 0, 3, 3, 3, 3, 3, 0, 0, 6, 6, 0, 0, 3, 3, 3, 3, 3, 0, 0, 6, 6, 0, 0, 0, 3, 0, 0, 0, 6, 6, 6, 0, 6, 0, 6, 6},
    {0, 0, 3, 5, 3, 0, 0, 5, 3, 5, 5, 3, 5, 5, 3, 5, 0, 3, 5, 3, 0, 0, 0, 3, 5, 3, 0, 0, 3, 5, 3, 0, 0, 0, 3, 5, 3, 0, 5, 3, 5, 5, 3, 5, 5, 3, 5, 0, 0, 3, 5, 3, 0, 0},
    {0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0, 4, 0, 0, 0},
    {5, 5, 5, 5, 5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 5, 5, 5, 5, 0, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5},
    {3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3, 0, 6, 6, 6, 0, 3, 3, 3, 3, 3, 3, 0, 6, 6, 6, 0, 3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3, 0, 3, 3, 3},
    {0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 0, 6, 0, 0, 6, 0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 6, 0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 6, 0, 0, 6, 0, 0, 6, 0, 0, 6, 0, 0, 0, 6, 0, 6, 0, 0}};

 for( int i = 0 ; i < 6 ; i++ )
  SteinA[i] = InitSteinA[i];

 for( int i = 0 ; i < 54 ; i++ )
 {
  for( int j = 0 ; j < 12 ; j++ )
  SteinFarbenLevel[j][i] = InitSteinFarbenLevel[j][i];

  for( int j = 0 ; j < 6 ; j++ )
  {
   SteinB[j][i] = InitSteinB[j][i];
   SteinXmin[j][i] = InitSteinXmin[j][i];
   SteinXmax[j][i] = InitSteinXmax[j][i];
  };

  SteinXpos[i] = InitSteinXpos[i];
  SteinYpos[i] = InitSteinYpos[i];
  SteinOrientierung[i] = InitSteinOrientierung[i];
  SteinMinX[i] = InitSteinMinX[i];
  SteinMaxX[i] = InitSteinMaxX[i];
  SteinMinY[i] = InitSteinMinY[i];
  SteinMaxY[i] = InitSteinMaxY[i];

 };
}

void GameField::StartGame()
{
 int ModLevel;

 if ( GameMode == 2 )	//game is already running
  return;

 if (GameMode == 1)	  //resume from pause
 {
  GameMode = 2;
  emit setgamemode(2);
  startTimer( 30 );
  return;
 };

  // Start new game
 GameMode=2;
 emit setgamemode(2);

 for( int i = 0 ; i < 3 ; i++ )
 {
  PlayerPoints[i] = 0;
	PadMode[i] = 0;
 };
 AnzBalls = 5;
 balls->display( AnzBalls );

 GameLevel = 1;
 level->display( GameLevel );

 ModLevel = GameLevel % 12;
 ModLevel--;
 for( int i = 0 ; i < 54 ; i++ )
  SteinFarbenAktuell[i] = SteinFarbenLevel[ModLevel][i];

 for( int i = 0 ; i < AnzPlayer ; i++)
  {
   PadMode[i] = 1;	// Pads coming in
   AnimationCounter[i] = 60;
  };

 BallMode = 1;		// Ball together with Pad1

 RefreshStones = true;
 ServeBall = false;

 startTimer( 30 );
}

void GameField::StopGame()
{
 killTimers();
 GameMode = 0;
 emit setgamemode(0);
 SetPlayer( AnzPlayer );	// set gamefield to start
}

void GameField::PauseGame()
{
 if ( GameMode < 2 )	//game is not running
  return;
 killTimers();
 GameMode = 1;
 emit setgamemode(1);
}


QPoint GameField::CalcRotPos( QPoint Position , int Schritte )
{
 // Rotiert die Position um Schritte * 60 im Uhrzeigersinn
 int PosX;
 int PosY;
 int RotPosX = 0;
 int RotPosY = 0;

 PosX = (Position.x()-120);
 PosY = (Position.y()-105);

 switch( Schritte )
 {
  case 1:
		// 60 
		RotPosX = ( PosX * 500 + PosY * 866 ) / 1000;
		RotPosY = ( -PosX * 866 + PosY * 500 ) / 1000;
	  break;
  case 2:
		// 120 
		RotPosX = ( -PosX * 500 + PosY * 866 ) / 1000;
		RotPosY = ( -PosX * 866 - PosY * 500 ) / 1000;
	  break;
  case 3:
		// 180 
		RotPosX = -PosX;
		RotPosY = -PosY;
	  break;
  case 4:
		// 240 
		RotPosX = ( -PosX * 500 - PosY * 866 ) / 1000;
		RotPosY = ( PosX * 866 - PosY * 500 ) / 1000;
	  break;
  case 5:
		// 300  = -60 
		RotPosX = ( PosX * 500 - PosY * 866 ) / 1000;
		RotPosY = ( PosX * 866 + PosY * 500 ) / 1000;
	  break;
 };

 return QPoint( RotPosX + 120 , RotPosY + 105 );

}

QPoint GameField::CalcCompPad( QPoint BallPos, QPoint OldPadPos )
{
 // Berechnet die Position des oberen Pads in Abhngigkeit von der Ballposition
 int bX;
 int pX;
 int pY;

 pX = OldPadPos.x();
 bX = BallPos.x();

 for( int i=0 ; i<3 ; i++ )   // Je mehr Schritt, desto Schneller bewegt sich das Computer-Pad
 {
    if (bX > pX) pX++;
    if (bX < pX) pX--;
 };

 if( pX < 72 )
		pX = 72;

 if( pX > 168 )
		pX = 168;

 pY = 40 - (pX-120)*(pX-120)/128;

 return QPoint(pX, pY);

}



int GameField::SetStones()
{
 // BitBlits the stones concerning to SteinFarbenAktuell from stoneimage into gameimage
 int Number = 0;
 //Alle Steine lschen
 bitBlt(&gameimage, 96, 84, &stonecopy, 0, 0, 48, 43, CopyROP , true );
 for( int i = 0 ; i < 54 ; i++ )
 {
  if( SteinFarbenAktuell[i] > 0 )
  {
   Number++;
   //Stein zeichnen
   bitBlt(&gameimage, SteinXpos[i], SteinYpos[i], &stoneimage, 8 * SteinOrientierung[i], 8 * SteinFarbenAktuell[i] - 8, 8, 8, CopyROP , false );
  };
 };
return Number;
}


void GameField::spacepressed()
{
 if( BallMode != 1 || PadMode[0] != 3 || GameMode != 2 )
 	return;
 ServeBall = true;
}


void GameField::mousePressEvent ( QMouseEvent * )
{
 if( BallMode != 1 || PadMode[0] != 3 || GameMode != 2 )
 	return;
 ServeBall = true;
}




void GameField::mouseMoveEvent( QMouseEvent *e )
{
 static bool CursorBlank;
 bool NewCursor;

 if( GameMode != 2 ) return;

 PadMoveTo = CalcPadPosition(e -> pos() );

 NewCursor = true;
 if( e -> x() < 30 || e -> x() > 210 || e -> y() < 140 || e -> y() > 205)
   NewCursor = false;

 if( NewCursor != CursorBlank )
 {
  if( NewCursor == false )
   setCursor(QCursor(0));                 // Mousecursor is set. blank=10 cross=2 normal=0
  if( NewCursor == true )
   setCursor(QCursor(10));
  CursorBlank = NewCursor;
 };

}


void GameField::PadMovement( QPoint Vektor )
{
	PadSpeed = Vektor;
}



void GameField::SetPadPosition()
{
 QPoint PadPosition;

 PadPosition = CalcPadPosition( PadMoveTo + QPoint(11,11) + PadSpeed);
 PadMoveTo = PadPosition;

 PadPosX[0] = PadPosition.x();
 PadPosY[0] = PadPosition.y();
}


QPoint GameField::CalcPadPosition( QPoint MousePosition )
{
 QPoint PadPosition;
 int X;
 int Y;
 float MaxY;
 float Steigung = 1.732050808; // tan(60)

 X = MousePosition.x();
 Y = MousePosition.y();

 X = X - 11;	// set center of pad to the mouses hotspot
 Y = Y - 11;

 //X-Wert nach links und rechts, Y-Wert nach unten eingrenzen
 if( X <= 46 )
 {
  //Pad links auerhalb
  X = 46;
  Y = 152;
 };

 if( X > 46 && X < 60 )
 {
  //Pad an linker Bande
  MaxY = Steigung * static_cast<float>(X) + 70.5;
  if( Y > MaxY )
  	Y = static_cast<int>(MaxY);
  if( Y > 178 )
  	Y = 178;
  if( X < 46 )
  	X = 46;
 };

 if( X >= 60 && X <= 76 && Y > 178 )   //Pad an linker Unterkante
 	Y = 178;

 if( X == 77 && Y > 180 )   //Pad am linken Pfosten
  Y = 180;

 if( X > 77 && X <= 138 && Y > 181 )   //Pad im Torbereich
  Y = 181;

 if( X == 139 && Y > 180 )  //Pad am rechten Pfosten
  Y = 180;

 if( X > 139 && X <= 158 && Y > 178 )	//Pad an rechter Unterkante
  Y = 178;

 if( X > 158 && X <= 178 )
 {
  //Pad an rechter Bande
  MaxY = -Steigung * static_cast<float>(X) + 453;
  if( Y > MaxY )
  	Y = static_cast<int>(MaxY);
  if( Y > 178 )
  	Y = 178;
  if( X > 171 )
  	X = 171;
 };

 if( X > 178 )	//Pad rechts auerhalb
 {
  X = 171;
  Y = 152;
 };

 //Y-Wert nach oben eingrenzen
 if( Y < 152 )
  Y = 152;

 PadPosition.setX( X );
 PadPosition.setY( Y );

 return PadPosition;
}



void GameField::timerEvent( QTimerEvent * )
{
 // is called in normal GameMode at 50Hz

 float Steigung = 1.732050808;
 QPoint BallPosition;
 int ModLevel;
 int i;
 int j;

 QPoint Rotated;
 QPoint BackRot;
 QPoint KIPadPos;
 static int RotPosX[2];
 static int RotPosY[2];

 float NewX;
 float NewY;
 float OldX;
 float OldY;
 float MinX;
 float MaxX;
 float MinY;
 float MaxY;
 float aBall;
 float bBall;
 float CollisionDistance;
 int CollisionKind;
 float CollisionParameter[4];
 float AbsSpeed;
 float TempSpeed;
 float s1;
 float s2;
 float s3;
 float aLot = 1;
 float bLot = 1;
 float xLot = 0;
 float yLot = 0;
 float xSchnitt = 0;
 float ySchnitt = 0;
 float BalldXrelative;
 float BalldYrelative;

 // Load Sounds
//static QSound SoundPad("/home/QtPalmtop/share/games/hexatrolic/hexatrolic_pad.wav"); // Sounds < 8k funktionieren nicht!
//static QSound SoundWall("/home/QtPalmtop/share/games/hexatrolic/hexatrolic_wall.wav");
//static QSound SoundStone("/home/QtPalmtop/share/games/hexatrolic/hexatrolic_stone.wav");


  if( GameMode != 2 )
 	{
	 killTimers();
 	 return;
	};

 switch (PadMode[0])  // Set Blue Pad
 {
	case 1:			//Pad fhrt ins Spielfeld
    AnimationCounter[0]--;
    PadPosX[0] = 109;
    PadPosY[0] = 159 + AnimationCounter[0];

		if( AnimationCounter[0] <= 0 )
    {
      PadMoveTo= QPoint(PadPosX[0],PadPosY[0]);
    	PadMode[0] = 3;
    	AnimationCounter[0] = 0;
    	ServeBall = false;
    };
  	break;
  case 2:      // Blue moves out
		AnimationCounter[0]--;
    if( AnimationCounter[0] == 149 )
    {
		  BallMode = 0;
      //bitBlt( &gameimage, static_cast<int>( BallPosX_old ), static_cast<int>( BallPosY_old ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
      bitBlt( &gameimage, static_cast<int>( BallPosX ), static_cast<int>( BallPosY ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
		};
    if( AnimationCounter[0] > 80 )
		{
      if( PadPosX[0] < 109 )
				PadPosX[0]++;
      if( PadPosX[0] > 109 )
				PadPosX[0]--;
      if( PadPosX[0] == 109 )
				AnimationCounter[0] = 80;
		};
    if( AnimationCounter[0] < 80 )
		{
	    if( PadPosY[0] < 210 )
				PadPosY[0]++;
    };
		if( AnimationCounter[0] <= 0 )
		{
			if( AnzBalls >= 1)
			{
  			ServeBall = false;
        BallMode = 1;
        PadMode[0] = 1;
  		  AnimationCounter[0] = 60;
        LastPadTouch = 0;
			}
			else
			{
  			GameMode = 0;
  			emit setgamemode(-1);
			};
		};
  	break;
 case 3:
  	SetPadPosition();
  	break;
 };

 switch (PadMode[1])	 // Set Red Pad
 {
  case 0:
  	break;
	case 1:            // Red Pad moves in
  	if( AnzPlayer == 2 )
  	{
      AnimationCounter[1]--;
      PadPosX[1] = 109;
      PadPosY[1] = 27 - AnimationCounter[1];

  		if( AnimationCounter[1] <= 0 )
      {
      	PadMode[1] = 4;
      	AnimationCounter[1] = 0;
				if ( BallMode == 2 ) ServeBall = true;
      };
  	};
  	if( AnzPlayer == 3 )
  	{
  	  AnimationCounter[1]--;
      PadPosX[1] = (64000 - 866 * AnimationCounter[1])/1000 -11;
      PadPosY[1] = (72000 - 500 * AnimationCounter[1])/1000 -11;

  		if( AnimationCounter[1] <= 0 )
      {
      	PadMode[1] = 4;
      	AnimationCounter[1] = 0;
			  if ( BallMode == 2 ) ServeBall = true;
  	  };
  	};
  	break;
  case 2:      // Red Pad moves out
    AnimationCounter[1]--;
    if( AnimationCounter[1] == 149 )
    {
		  BallMode = 0;
      //bitBlt( &gameimage, static_cast<int>( BallPosX_old ), static_cast<int>( BallPosY_old ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
			bitBlt( &gameimage, static_cast<int>( BallPosX ), static_cast<int>( BallPosY ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
			if( AnzPlayer == 3 ) {
  			Rotated = CalcRotPos( QPoint( PadPosX[1] + 11 , PadPosY[1] + 11 ) , 2 );
			}
			else {
  			Rotated = CalcRotPos( QPoint( PadPosX[1] + 11 , PadPosY[1] + 11 ) , 3 );
			};
			RotPosX[0] = Rotated.x() - 11 ;
      RotPosY[0] = Rotated.y() - 11;
		};
    if( AnimationCounter[1] > 80 )
		{
      if( RotPosX[0] < 109 )
				RotPosX[0]++;
      if( RotPosX[0] > 109 )
				RotPosX[0]--;
      if( RotPosX[0] >= 107 && RotPosX[0] <= 111 )
				AnimationCounter[1] = 80;
		};
    if( AnimationCounter[1] < 80 )
		{
	    if( RotPosY[0] < 220 )
				RotPosY[0]++;
    };
		if( AnimationCounter[1] <= 0 )
		{
			if( AnzBalls >= 1)
			{
  			ServeBall = false;
        BallMode = 2;
        PadMode[1] = 1;
  		  AnimationCounter[1] = 60;
        LastPadTouch = 1;
			};
		};

    if( AnzPlayer == 3 ) {
  		BackRot = CalcRotPos( QPoint( RotPosX[0]+11 , RotPosY[0]+11 ) , 4 );
		}
		else {
  		BackRot = CalcRotPos( QPoint( RotPosX[0]+11 , RotPosY[0]+11 ) , 3 );
		};

	  PadPosX[1] = BackRot.x()-11;
    PadPosY[1] = BackRot.y()-11;

  	break;
  case 4:
   if( AnzPlayer == 3 ) {
   		KIPadPos = CalcRotPos(CalcCompPad( CalcRotPos( QPoint( static_cast<int>(BallPosX)+4,static_cast<int>(BallPosY)+4),5), CalcRotPos( QPoint( PadPosX[1]+11, PadPosY[1]+11 ),5)),1);

	  }
	  else {
   		KIPadPos = CalcCompPad( QPoint( static_cast<int>(BallPosX)+4,static_cast<int>(BallPosY)+4) , QPoint( PadPosX[1]+11, PadPosY[1]+11 ) );
  		//KIPadPos = CalcRotPos(CalcCompPad( CalcRotPos( QPoint( static_cast<int>(BallPosX)+4,static_cast<int>(BallPosY)+4),3), CalcRotPos( QPoint( PadPosX[1]+11, PadPosY[1]+11 ),3)),3);
  		//KIPadPos = CalcCompPad( QPoint( static_cast<int>(BallPosX)-4,static_cast<int>(BallPosY)-4) );

    };
    PadPosX[1] =  KIPadPos.x()-11;
    PadPosY[1] =  KIPadPos.y()-11;

  	break;
 };

 switch (PadMode[2]) // Set Green Pad
 {
  case 0:
  	break;
	case 1:
  		AnimationCounter[2]--;
      PadPosX[2] = (176000 + 866 * AnimationCounter[2])/1000 -11;
      PadPosY[2] = (72000 - 500 * AnimationCounter[2])/1000 -11;

  		if( AnimationCounter[2] <= 0 )
      {
      	PadMode[2] = 4;
      	AnimationCounter[2] = 0;
			if ( BallMode == 3 ) ServeBall = true;
  	  };
  	break;
  case 2:
    AnimationCounter[2]--;
    if( AnimationCounter[2] == 149 )
    {
		  BallMode = 0;
      //bitBlt( &gameimage, static_cast<int>( BallPosX_old ), static_cast<int>( BallPosY_old ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
			bitBlt( &gameimage, static_cast<int>( BallPosX ), static_cast<int>( BallPosY ), &ballcopy, 0, 0, 8, 8, CopyROP, true );
			Rotated = CalcRotPos( QPoint( PadPosX[2] + 11 , PadPosY[2] + 11 ) , 4 );
      RotPosX[1] = Rotated.x() - 11 ;
      RotPosY[1] = Rotated.y() - 11;
		};
		if( AnimationCounter[2] > 80 )
		{
      if( RotPosX[1] < 109 )
				RotPosX[1]++;
      if( RotPosX[1] > 109 )
				RotPosX[1]--;
      if( RotPosX[1] >= 107 && RotPosX[1] <= 111 )
				AnimationCounter[2] = 80;
		};
    if( AnimationCounter[2] < 80 )
		{
	    if( RotPosY[1] < 220 )
				RotPosY[1]++;
    };

		if( AnimationCounter[2] <= 0 )
		{
			if( AnzBalls >= 1)
			{
  			ServeBall = false;
        BallMode = 3;
        PadMode[2] = 1;
  		  AnimationCounter[2] = 60;
        LastPadTouch = 2;
			};
		};
		BackRot = CalcRotPos( QPoint( RotPosX[1] + 11 , RotPosY[1] + 11 ) , 2 );
	  PadPosX[2] = BackRot.x() - 11;
    PadPosY[2] = BackRot.y() - 11;
    break;
  case 4:
 		KIPadPos = CalcRotPos(CalcCompPad( CalcRotPos( QPoint( static_cast<int>(BallPosX)+4,static_cast<int>(BallPosY)+4),1), CalcRotPos( QPoint( PadPosX[2]+11, PadPosY[2]+11 ),1)),5);


	  PadPosX[2] =  KIPadPos.x()-11;
    PadPosY[2] =  KIPadPos.y()-11;

  	break;
 };


 // Set Ball
 switch (BallMode)
 {
  case 0:
  	break;
	case 1:	// Ball an Pad1
		BallPosition = CalcBallPosition(0);
    BallPosX = BallPosition.x();
    BallPosY = BallPosition.y();
    if( ServeBall )
  	{
  		//Aufschlag!
  		BalldX = BallPosX - PadPosX[0] - 7;
      BalldY = BallPosY - PadPosY[0] - 7;
      BallSpeedX = (BalldX / 5) * static_cast<float>((10+GameLevel))/10;
      BallSpeedY = (BalldY / 5) * static_cast<float>((10+GameLevel))/10;
      BallMode = 7;
  	};
  	break;
  case 2:	// Ball an Pad2
		BallPosition = CalcBallPosition(1);
    BallPosX = BallPosition.x();
    BallPosY = BallPosition.y();
   if( ServeBall )
  	{
  		//Aufschlag!
  		BalldX = BallPosX - PadPosX[1] - 7;
      BalldY = BallPosY - PadPosY[1] - 7;
      BallSpeedX = (BalldX / 5) * static_cast<float>((10+GameLevel))/10;
      BallSpeedY = (BalldY / 5) * static_cast<float>((10+GameLevel))/10;
      BallMode = 7;
  	};
  break;
  case 3:	// Ball an Pad3
		BallPosition = CalcBallPosition(2);
    BallPosX = BallPosition.x();
    BallPosY = BallPosition.y();
  if( ServeBall )
  	{
  		//Aufschlag!
  		BalldX = BallPosX - PadPosX[2] - 7;
      BalldY = BallPosY - PadPosY[2] - 7;
      BallSpeedX = (BalldX / 5) * static_cast<float>((10+GameLevel))/10;
      BallSpeedY = (BalldY / 5) * static_cast<float>((10+GameLevel))/10;
      BallMode = 7;
  	};
  break;
  case 4:	// Ball unterwegs zu Pad1
		BallPosition = CalcBallPosition(0);
		if (BallPosX < BallPosition.x()) BallPosX++;
		if (BallPosX > BallPosition.x()) BallPosX--;
    if (BallPosY < BallPosition.y()) BallPosY++;
    if (BallPosY > BallPosition.y()) BallPosY--;

    if( fabs(BallPosX - BallPosition.x()) <= 2 && fabs(BallPosY - BallPosition.y()) <= 2 )
     {
     //Ball nahe genug dran!
      GameLevel = GameLevel + 1;
      ModLevel = GameLevel % 12;
      ModLevel--;
      for( i = 0 ; i < 54 ; i++ )
       SteinFarbenAktuell[i] = SteinFarbenLevel[ModLevel][i];
      level->display(GameLevel);
      RefreshStones = true;
			ServeBall = false;
      BallMode = 1;
      AnimationCounter[0] = 0;

      BallPosX = BallPosition.x();
      BallPosY = BallPosition.y();
    };

    break;
	case 5:	// Ball unterwegs zu Pad2
  	BallPosition = CalcBallPosition(1);
		if (BallPosX < BallPosition.x()) BallPosX++;
		if (BallPosX > BallPosition.x()) BallPosX--;
    if (BallPosY < BallPosition.y()) BallPosY++;
    if (BallPosY > BallPosition.y()) BallPosY--;

    if( fabs(BallPosX - BallPosition.x()) <= 2 && fabs(BallPosY - BallPosition.y()) <= 2 )
     {
     //Ball nahe genug dran!
      GameLevel = GameLevel + 1;
      ModLevel = GameLevel % 12;
      ModLevel--;
      for( int i = 0 ; i < 54 ; i++ )
       SteinFarbenAktuell[i] = SteinFarbenLevel[ModLevel][i];
      level->display(GameLevel);
      RefreshStones = true;
      AnimationCounter[1] = 0;
      BallPosX = BallPosition.x();
      BallPosY = BallPosition.y();
      BalldX = BallPosX - PadPosX[1] - 7;
      BalldY = BallPosY - PadPosY[1] - 7;
      BallSpeedX = (BalldX / 5) * (1+GameLevel/5);
      BallSpeedY = (BalldY / 5) * (1+GameLevel/5);;
      BallMode = 7;

    };
  	break;
  case 6:	// Ball unterwegs zu Pad3
		BallPosition = CalcBallPosition(2);
		if (BallPosX < BallPosition.x()) BallPosX++;
		if (BallPosX > BallPosition.x()) BallPosX--;
    if (BallPosY < BallPosition.y()) BallPosY++;
    if (BallPosY > BallPosition.y()) BallPosY--;

    if( fabs(BallPosX - BallPosition.x()) <= 2 && fabs(BallPosY - BallPosition.y()) <= 2 )
     {
     //Ball nahe genug dran!
      GameLevel = GameLevel + 1;
      ModLevel = GameLevel % 12;
      ModLevel--;
      for( int i = 0 ; i < 54 ; i++ )
       SteinFarbenAktuell[i] = SteinFarbenLevel[ModLevel][i];
      level->display(GameLevel);
			RefreshStones = true;
      AnimationCounter[2] = 0;

      BallPosX = BallPosition.x();
      BallPosY = BallPosition.y();

			BalldX = BallPosX - PadPosX[2] - 7;
      BalldY = BallPosY - PadPosY[2] - 7;
      BallSpeedX = (BalldX / 5) * (1+GameLevel/5);
      BallSpeedY = (BalldY / 5) * (1+GameLevel/5);;
      BallMode = 7;

    };
  	break;
  case 7:
        /*----------------------------------------------------
           Freie Ballbewegung
        ----------------------------------------------------*/

    BalldX = BallSpeedX;
    BalldY = BallSpeedY;
    OldX = BallPosX;
    OldY = BallPosY;
		AbsSpeed = sqrt(BallSpeedX * BallSpeedX + BallSpeedY * BallSpeedY);


    do
    {
	    //Berechnung der Gradengleichung fr die Ballbewegung
      if( BalldX != 0 )
        {
        aBall = BalldY / BalldX;
        bBall = OldY - aBall * OldX;
        }
      else //Funktion nicht definiert!
        {
        aBall = 0;
        bBall = 0;
        };
 			//Berechnen der neuen Ball Position
      NewX = OldX + BalldX;
      NewY = OldY + BalldY;
			if( NewX > OldX ) {
				MaxX = NewX;
				MinX = OldX;
			}
			else
			{
				MinX = NewX;
				MaxX = OldX;
			};
			if( NewY > OldY ) {
				MaxY = NewY;
				MinY = OldY;
			}
			else
			{
				MinY = NewY;
				MaxY = OldY;
			};


      /*----------------------------------------------------
           Kontaktberprfung initialisieren!
        ----------------------------------------------------*/
			CollisionKind = 0;
      CollisionDistance = 57600;   //=240^2-> grer als das Spielfeld

      /*----------------------------------------------------
           Kontakt mit Bande berprfen!
        ----------------------------------------------------*/

   		//Oben links
      if( NewX <= 62 && NewY < 116 - Steigung * NewX && aLot != -Steigung )
      {
          //Kollision mit oberer linker Bande
          //Gradengleichung des Lotes bestimmen
          aLot = 1 / Steigung;
          bLot = NewY - aLot * NewX;
          //Schnittpunkt zwischen Lot und Bande
          xLot = (116 - bLot) / (aLot + Steigung);
          yLot = aLot * xLot + bLot;
          //Schnittpunkt Ballbahn und Band -> Kontakt!
          if( BalldX == 0 ) {
              xSchnitt = OldX;
          }
          else {
              xSchnitt = (116 - bBall) / (aBall + Steigung);
          };
          ySchnitt = -Steigung * xSchnitt + 116;
          //Kollisionsdistanz berechnen
          s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) * (ySchnitt - OldY);
          if( s1 < CollisionDistance )
					{
					  CollisionDistance = s1;
						if( AnzPlayer == 3 && xSchnitt >= 20 && xSchnitt <=54 )
						{
  						// Tor 3
              CollisionKind = 5;
   						CollisionParameter[0] = NewX;
   						CollisionParameter[1] = NewY;
   						CollisionParameter[2] = xSchnitt;
   						CollisionParameter[3] = ySchnitt;
						}
						else
						{
  						CollisionKind = 2;
  						CollisionParameter[0] = 2 * xLot - NewX;
  						CollisionParameter[1] = 2 * yLot - NewY;
  						CollisionParameter[2] = xSchnitt;
  						CollisionParameter[3] = ySchnitt;
						};
          };
       };

       //Unten links
       if( NewX <= 62 && NewY > Steigung * NewX + 88 && aBall != Steigung )
       {
           //Kollision mit unterer linker Bande
           //Gradengleichung des Lotes bestimmen
           aLot = -1 / Steigung;
           bLot = NewY - aLot * NewX;
           //Schnittpunkt zwischen Lot und Bande
           xLot = (88 - bLot) / (aLot - Steigung);
           yLot = aLot * xLot + bLot;
           //Schnittpunkt Ballbahn und Band -> Kontakt!
           if( BalldX == 0 ) {
               xSchnitt = OldX;
           }
           else {
               xSchnitt = (88 - bBall) / (aBall - Steigung);
           };
           ySchnitt = Steigung * xSchnitt + 88;
           //Kollisionsdistanz berechnen
           s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) * (ySchnitt - OldY);
           if( s1 < CollisionDistance )
					 {
              //NewX = 2 * xLot - NewX;
              //NewY = 2 * yLot - NewY;
              CollisionDistance = s1;
   						CollisionKind = 2;
							CollisionParameter[0] = 2 * xLot - NewX;
  						CollisionParameter[1] = 2 * yLot - NewY;
  						CollisionParameter[2] = xSchnitt;
  						CollisionParameter[3] = ySchnitt;
   				 };
       };

			//Oben rechts
      if( NewX >= 170 && NewY < Steigung * NewX - 286 && aBall != Steigung )
      {
          //Kollision mit oberer rechter Bande
			    //Gradengleichung des Lotes bestimmen
          aLot = -1 / Steigung;
          bLot = NewY - aLot * NewX;
          //Schnittpunkt zwischen Lot und Bande
          xLot = (-286 - bLot) / (aLot - Steigung);
          yLot = aLot * xLot + bLot;
          //Schnittpunkt Ballbahn und Band -> Kontakt!
          if( BalldX == 0 ) {
              xSchnitt = OldX;
          }
          else {
              xSchnitt = (-286 - bBall) / (aBall - Steigung);
          };
          ySchnitt = Steigung * xSchnitt + -286;
          //Kollisionsdistanz berechnen
          s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) * (ySchnitt - OldY);
          if( s1 < CollisionDistance )
					{
            CollisionDistance = s1;
						if( AnzPlayer == 3 && xSchnitt >= 186 && xSchnitt <= 220 )
						{
  						// Tor 4
              CollisionKind = 6;
   						CollisionParameter[0] = NewX;
   						CollisionParameter[1] = NewY;
   						CollisionParameter[2] = xSchnitt;
   						CollisionParameter[3] = ySchnitt;
						}
						else
						{
  						CollisionKind = 2;
  						CollisionParameter[0] = 2 * xLot - NewX;
  						CollisionParameter[1] = 2 * yLot - NewY;
  						CollisionParameter[2] = xSchnitt;
  						CollisionParameter[3] = ySchnitt;
						};
          };
      };

			//Unten rechts
      if( NewX >= 170 && NewY > 488.0 - Steigung * NewX && aBall != -Steigung )
      {
          //Kollision mit unterer rechter Bande
			    //Gradengleichung des Lotes bestimmen
          aLot = 1.0 / Steigung;
          bLot = NewY - aLot * NewX;
          //Schnittpunkt zwischen Lot und Bande
          xLot = (488.0 - bLot) / (aLot + Steigung);
          yLot = -Steigung * xLot + 488.0;
          //yLot = aLot * xLot + bLot;
          //Schnittpunkt Ballbahn und Band -> Kontakt!
          if( BalldX == 0 ) {
              xSchnitt = OldX;
					}
          else {
              xSchnitt = (488 - bBall) / (aBall + Steigung);
          };
					ySchnitt = -Steigung * xSchnitt + 488;
          //Kollisionsdistanz berechnen
          s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) * (ySchnitt - OldY);
          if( s1 < CollisionDistance  )
					{
              //NewX = 2 * xLot - NewX;
              //NewY = 2 * yLot - NewY;
              CollisionDistance = s1;
							CollisionKind = 2;
  						CollisionParameter[0] = 2 * xLot - NewX;
  						CollisionParameter[1] = 2 * yLot - NewY;
  						CollisionParameter[2] = xSchnitt;
  						CollisionParameter[3] = ySchnitt;
          };
      };

      //Oben
      if( NewY + 1 < 10 && BalldY != 0  )
			{
			    //Kollision mit oberer Bande
          ySchnitt = 9;
          if( BalldX == 0 || aBall == 0 ) {
              xSchnitt = OldX;
          }
          else {
              xSchnitt = (9 - bBall) / aBall;
					};
          //Kollisionsdistanz berechnen
          s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) * (ySchnitt - OldY);
          if( s1 < CollisionDistance )
          {
              CollisionDistance = s1;
							//durchs Tor gespielt?
              if( xSchnitt >= 83 && xSchnitt <= 151 && AnzPlayer == 2)
              {
                  //Tor 2
                  CollisionKind = 4;
      						CollisionParameter[0] = NewX;
      						if( NewX < 83 )
                  	CollisionParameter[0] = 83;
                  if( NewX > 149 )
                  	CollisionParameter[0] = 149;
                  CollisionParameter[1] = NewY;
      						CollisionParameter[2] = xSchnitt;
      						CollisionParameter[3] = ySchnitt;
							}
              else
							{
                //Reflektion
                CollisionKind = 2;
    						CollisionParameter[0] = NewX;
    						CollisionParameter[1] = 18 - NewY;
    						CollisionParameter[2] = xSchnitt;
    						CollisionParameter[3] = ySchnitt;
							};
          };
      };

      //Unten
      if( NewY + 7 > 200 && BalldY != 0  )
			{
			    //Kollision mit unterer Bande
          ySchnitt = 193;
          if( BalldX == 0 ) {
              xSchnitt = OldX;
          }
          else {
              xSchnitt = (193 - bBall) / aBall;
          };
          //Kollisionsdistanz berechnen
          s1 = (xSchnitt - OldX) * (xSchnitt - OldX) + (ySchnitt - OldY) *(ySchnitt - OldY);
          if( s1 < CollisionDistance )
          {
              CollisionDistance = s1;
							//durchs Tor gespielt?
              if( xSchnitt >= 83 && xSchnitt <= 151 )
              {
                  //Tor!
                  CollisionKind = 3;
      						CollisionParameter[0] = NewX;
      						if( NewX < 83 )
                  	CollisionParameter[0] = 83;
                  if( NewX > 149 )
                  	CollisionParameter[0] = 149;
                  CollisionParameter[1] = NewY;
      						CollisionParameter[2] = xSchnitt;
      						CollisionParameter[3] = ySchnitt;
							}
              else
							{
                  //Reflektion
                  //NewY = 386 - NewY;
                  CollisionKind = 2;
      						CollisionParameter[0] = NewX;
      						CollisionParameter[1] = 386 - NewY;
      						CollisionParameter[2] = xSchnitt;
      						CollisionParameter[3] = ySchnitt;
              };
          };
      };


				/*----------------------------------------------------
           Kontakt mit Steinen berprfen!
        ----------------------------------------------------*/

       if( !(MaxX < 89 || MinX > 143 || MaxY < 77 || MinY > 125) )
			 {
           //Ball im Bereich der Steine
					 for( i = 0 ; i < 54 ; i++ )
					 {
               //Fr jeden Stein...
               if( SteinFarbenAktuell[i] > 0 )
               {
                   //... der vorhanden ist ...
                   if( !(MaxX < SteinMinX[i] || MinX > SteinMaxX[i] || MaxY < SteinMinY[i] || MinY > SteinMaxY[i]))
                       //... der im Bereich des Balles ist ...
									 {
                       for( j = 0 ; j < 6 ; j++ )
                       {
                           //... und jede den Stein begrenzende Gerade
                           if( BalldX != 0 )
                           {
                               if( fabs(aBall - SteinA[j]) < 0.00000001 && fabs(SteinB[j][i] - bBall) < 0.00000001 )
                               {
                                   //Ballgrade und Steingrade stimmen berein!
                                   if( BalldX > 0 ) {
                                       xSchnitt = SteinXmin[j][i];
                                   }
                                   else {
                                       xSchnitt = SteinXmax[j][i];
                                   };
                               }
                               else {
                                   xSchnitt = (SteinB[j][i] - bBall) / (aBall - SteinA[j]);
                               };
                           }
                           else {
                               //Ball fliegt senkrecht!
                               xSchnitt = OldX;
                           };

                           ySchnitt = SteinA[j] * xSchnitt + SteinB[j][i];
                           s1 = (xSchnitt - OldX)*(xSchnitt - OldX) + (ySchnitt - OldY)*(ySchnitt - OldY);

                           if( xSchnitt >= SteinXmin[j][i] && xSchnitt <= SteinXmax[j][i] && s1 < CollisionDistance && xSchnitt >= MinX && xSchnitt <= MaxX && ySchnitt >= MinY && ySchnitt <= MaxY )
													 {
                               //neuer erster Schnittpunkt!
                               CollisionDistance = s1;
                               CollisionKind = 1;
            									 CollisionParameter[0] = static_cast<float>(i);
                  						 CollisionParameter[1] = static_cast<float>(j);
                  						 CollisionParameter[2] = xSchnitt;
                  						 CollisionParameter[3] = ySchnitt;

                           };
                       };
                   };
               };
           };
       };




      /*----------------------------------------------------
           Kontakt mit Pads berprfen!
        ----------------------------------------------------*/

      for( i = 0 ; i < 3 ; i++ )
      {

         if( !( PadMode[i] == false || (OldX < PadPosX[i] - 7 && NewX < PadPosX[i] - 7) || (OldX > PadPosX[i] + 21 && NewX > PadPosX[i] + 21) || (OldY < PadPosY[i] - 7 && NewY < PadPosY[i] - 7) || (OldY > PadPosY[i] + 21 && NewY > PadPosY[i] + 21)) )
         {
             //Ball im Bereich des Pads und Pad vorhanden
             if( (PadPosX[i] - NewX + 7) * (PadPosX[i] - NewX + 7) + (PadPosY[i] - NewY + 7) * (PadPosY[i] - NewY + 7) < 196 ) //((PadPosX[i]+11)-(newx+4))^2+...
						 {
             		 //Kontakt!

          			//Berechnung der relativen Gradengleichung fr die Ballbewegung, Das Koordinatensystem bewegt sich mit dem Pad
								BalldXrelative = BalldX - static_cast<float>( PadPosX[i] - PadPosX_old[i] );
								BalldYrelative = BalldY - static_cast<float>( PadPosY[i] - PadPosY_old[i] );

                 //Schnittpunkt berechnen
                 if( BalldXrelative != 0 )  						//if( BalldX != 0 )
                 {
                  	 aBall = BalldYrelative / BalldXrelative;
                  	 bBall = OldY - aBall * OldX;

                     s1 = bBall - PadPosY[i] - 4 * aBall - 7;
                     s2 = PadPosX[i] + 11;
                     s3 = ((aBall*s1-s2)/(aBall*aBall+1))*((aBall*s1-s2)/(aBall*aBall+1)) - (s1*s1 + s2*s2 - 225) / (aBall*aBall + 1);
                     if( s3 >= 0 )
                     {
                         //reales Ergebnis
                         s3 = sqrt(s3);
                         xSchnitt = (s2 - aBall * s1) / (aBall*aBall + 1);
                         if( BalldXrelative > 0 ) { 			//if( BalldX > 0 ) {
                             xSchnitt = xSchnitt - s3;
                         }
                         else {
                             xSchnitt = xSchnitt + s3;
                         };
                         ySchnitt = aBall * (xSchnitt - 4) + bBall + 4;
                     };
								 }
                 else
                 {
                     xSchnitt = NewX+3.9;
                     s3 = 225 - (xSchnitt - PadPosX[i] - 11)*(xSchnitt - PadPosX[i] - 11);
										 ySchnitt = PadPosY[i] + 11;
                     if( s3 < 0 ) {
												s3 = 0;
										 }
										 else {
                     		s3 = sqrt(s3);
										 };
                     if( BalldYrelative >= 0 ) {
                         ySchnitt = ySchnitt - s3;
										 }
                     else {
                         ySchnitt = ySchnitt + s3;
										 };

                 };

                 //Kollisionsdistanz berechnen (nur gltig fr s3>=0)
                 s1 = (xSchnitt - 4 - OldX)*(xSchnitt - 4 - OldX) + (ySchnitt - 4 - OldY)*(ySchnitt - 4 - OldY);

                 if( s3 >= 0 && s1 < CollisionDistance )
								 {
                     CollisionDistance = s1;

                     if( xSchnitt == PadPosX[i] + 11 || ySchnitt == PadPosY[i] + 11 )
                     {
                         //senkrechte oder waagerechte Reflektionsebene
                         if( ySchnitt < PadPosY[i] + 11 ) {
                             //Reflektion an Oberkante
                             CollisionParameter[1] = 2 * PadPosY[i] - NewY - 8;
														 CollisionParameter[0] = NewX;
												 }
                         else {
                         		if( ySchnitt > PadPosY[i] + 11 ) {
                             	//Reflektion an Unterkante
                              CollisionParameter[1] = 2 * PadPosY[i] - NewY + 36;
 	  													CollisionParameter[0] = NewX;
														}
														else {
														  if( xSchnitt < PadPosX[i] + 11 ) {
                                //Reflektion an linker Kante
                                CollisionParameter[0] = 2 * PadPosX[i] - NewX - 8;
																CollisionParameter[1] = NewY;
                              }
                              else {
                                  //Reflektion an rechter Kante
                                  CollisionParameter[0] = 2 * PadPosX[i] - NewX + 36;
																	CollisionParameter[1] = NewY;
                              };
                            };
                         };
                     }
                     else
                     {
                         // aLot=Steigung, Lot
                         // bLot=Y-Achsenabschnitt, Lot
                         // s1=Steigung, Senkrechte auf Lot, durch (newx; newy)
                         // s2=Y-Achsenabschnitt, Senkrechte auf Lot, durch (newx; newy)

                         //Lot-Grade Berechnen
                         aLot = (PadPosY[i] + 11 - ySchnitt) / (PadPosX[i] + 11 - xSchnitt);
                         bLot = ySchnitt - aLot * xSchnitt;
                         //Senkrechte durch Lot berechnen, die durch (newx; newy) geht
                         s1 = -1 / aLot;
                         s2 = (NewY + 4) - s1 * (NewX + 4);
                         //Schnittpunkt der Senkrechten durch Lot mit Lot bestimmen
                         xLot = (bLot - s2) / (s1 - aLot);
                         yLot = aLot * xLot + bLot;
                         //Reflektionspunkt bestimmen
                         CollisionParameter[0] = 2 * xSchnitt - 2 * xLot + NewX;
                         CollisionParameter[1] = 2 * ySchnitt - 2 * yLot + NewY;
										 };
                     xSchnitt = xSchnitt - 4;
                     ySchnitt = ySchnitt - 4;
                     CollisionKind = 7 + i;
   									 CollisionParameter[2] = xSchnitt;
         						 CollisionParameter[3] = ySchnitt;

                 };
             };
         };
      };


      switch (CollisionKind)
       {
    		case 0:
    			//Bewegung ohne Kollision beenden
         TempSpeed = AbsSpeed / sqrt(BalldX*BalldX + BalldY*BalldY);
         BallSpeedX = BalldX * TempSpeed;
         BallSpeedY = BalldY * TempSpeed;
         BalldX = 0;
         BalldY = 0;
 				break;
    		case 1:
 				// Kontakt mit Stein!
         i = static_cast<int>(CollisionParameter[0]);
         j = static_cast<int>(CollisionParameter[1]);
         xSchnitt = CollisionParameter[2];
         ySchnitt = CollisionParameter[3];

         if( SteinA[j] != 0 )
         {
             //Reflektion an schrger Grade
             //Lot durch (newx;newy) bestimmen
             aLot = -1 / SteinA[j];
             bLot = NewY - aLot * NewX;
             //Schnittpunkt zwischen Lot und Grade
             xLot = (SteinB[j][i] - bLot) / (aLot - SteinA[j]);
             yLot = aLot * xLot + bLot;
             //neuen virtuellen Zielpunkt berechnen
             NewX = 2 * xLot - NewX;
             NewY = 2 * yLot - NewY;
 				 }
         else
         {
             //Reflektion an waagerechter Grade
             NewY = 2 * SteinB[j][i] - NewY;
         };

         OldX = xSchnitt;
         OldY = ySchnitt;

         BalldX = NewX - xSchnitt;
         BalldY = NewY - ySchnitt;

         //Ballgeschwindigkeit um 1% vergrern (1,005^2 = 1,01)!
         //s2 = 1 + 0.005 / sqrt(BalldX*BalldX + BalldY*BalldY);
         //BalldX = BalldX * s2;
         //BalldY = BalldY * s2;
         //AbsSpeed = AbsSpeed * s2 * s2;
         AbsSpeed = AbsSpeed * 1.01;

         //Stein lschen
         SteinFarbenAktuell[i] = -1;
         RefreshStones = true;
 			 	PlayerPoints[LastPadTouch]++;


			  //SoundStone.play();

 				break;
 			case 2:
					//Kontakt mit Bande!
         NewX = CollisionParameter[0];
         NewY = CollisionParameter[1];
         xSchnitt = CollisionParameter[2];
         ySchnitt = CollisionParameter[3];

         OldX = xSchnitt;
         OldY = ySchnitt;
         BalldX = NewX - xSchnitt;
         BalldY = NewY - ySchnitt;

 				//SoundWall.play();

 				break;
 			case 3:
 				// Tor 1 (Pad Blue)    (Game Over wird bei der Padberechnung gesetzt)
 				AnzBalls --;
 				balls->display(AnzBalls);
 			  if( AnzPlayer == 2)
 				{
 					PlayerPoints [1] = PlayerPoints [1] + 100;
      		emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);
 				};
 			  if( AnzPlayer == 3)
 				{
 					PlayerPoints [1] = PlayerPoints [1] + 50;
 					PlayerPoints [2] = PlayerPoints [2] + 50;
 					emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);
 				};

 				TempSpeed = AbsSpeed / sqrt(BalldX*BalldX + BalldY*BalldY);
         BallSpeedX = BalldX * TempSpeed;
         BallSpeedY = BalldY * TempSpeed;
         BalldX = 0;
         BalldY = 0;
         PadMode[0]=2;							// Ball wird im nchsten Durchlauf gelscht! (BallMode=0)
	        AnimationCounter[0] = 150;
 				if( AnzBalls < 1)
 				{
           if( PadMode[1] > 0 )
 					{
 						//Pad 2 rausfahren, da Game Over!
 						PadMode[1] = 2;
 						AnimationCounter[1] = 150;
 					};
           if( PadMode[2] > 0 )
 					{
 						//Pad 3 rausfahren, da Game Over!
 						PadMode[2] = 2;
 						AnimationCounter[2] = 150;
 					};
 				};

    			break;
 			case 4:
 			case 5:
 				// Tor 2 (Pad Red)
   			   if( AnzPlayer == 2)
   				{
   					PlayerPoints [0] = PlayerPoints [0] + 100;
        			//emit setbluepoints(PlayerPoints[0]);
 						emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);

   				};
   			   if( AnzPlayer == 3)
   				{
   					PlayerPoints [0] = PlayerPoints [0] + 50;
   					PlayerPoints [2] = PlayerPoints [2] + 50;
        			emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);
 						// emit setbluepoints(PlayerPoints[0]);
   					// emit setgreenpoints(PlayerPoints[2]);
   				};
              TempSpeed = AbsSpeed / sqrt(BalldX*BalldX + BalldY*BalldY);
              BallSpeedX = BalldX * TempSpeed;
              BallSpeedY = BalldY * TempSpeed;
              BalldX = 0;
              BalldY = 0;
              PadMode[1]=2;							// Ball wird im nchsten Durchlauf gelscht! (BallMode=0)
 		        AnimationCounter[1] = 150;
 			break;
 			case 6:
 				// Tor 3 (Pad Green)
              PlayerPoints [0] = PlayerPoints [0] + 50;
              PlayerPoints [1] = PlayerPoints [1] + 50;
              emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);
 						 // emit setbluepoints(PlayerPoints[0]);
              // emit setredpoints(PlayerPoints[1]);
              TempSpeed = AbsSpeed / sqrt(BalldX*BalldX + BalldY*BalldY);
              BallSpeedX = BalldX * TempSpeed;
              BallSpeedY = BalldY * TempSpeed;
              BalldX = 0;
              BalldY = 0;
              PadMode[2]=2;							// Ball wird im nchsten Durchlauf gelscht! (BallMode=0)
              AnimationCounter[2] = 150;
 			break;
 			case 7:
 				//Kontakt mit Pad1 (Blue)
 				NewX = CollisionParameter[0];
        NewY = CollisionParameter[1];
        xSchnitt = CollisionParameter[2];
        ySchnitt = CollisionParameter[3];

        OldX = xSchnitt;
        OldY = ySchnitt;
        BalldX = NewX - xSchnitt;
        BalldY = NewY - ySchnitt;

	 			LastPadTouch = 0;
				//SoundPad.play();

 			break;
 			case 8:
 				//Kontakt mit Pad2 (Red)
 				NewX = CollisionParameter[0];
        NewY = CollisionParameter[1];
        xSchnitt = CollisionParameter[2];
        ySchnitt = CollisionParameter[3];

        OldX = xSchnitt;
        OldY = ySchnitt;
        BalldX = NewX - xSchnitt;
        BalldY = NewY - ySchnitt;

  			LastPadTouch = 1;
  			//SoundPad.play();

 			break;
 			case 9:
 				//Kontakt mit Pad3 (Green)
 				NewX = CollisionParameter[0];
        NewY = CollisionParameter[1];
        xSchnitt = CollisionParameter[2];
        ySchnitt = CollisionParameter[3];

        OldX = xSchnitt;
        OldY = ySchnitt;
        BalldX = NewX - xSchnitt;
        BalldY = NewY - ySchnitt;

 				LastPadTouch = 2;
  			//SoundPad.play();

 			break;
			};

    }while( BalldX != 0 || BalldY != 0 );

    BallPosX = NewX;
    BallPosY = NewY;

  	break;
 };


 MoveSprites();

}


QPoint GameField::CalcBallPosition( int PadNumber )
{
 int PosX = 0;
 int PosY = 0;
 QPoint Rotated;
 QPoint BackRot;

 switch(PadNumber)
 {
  case 0:
  	if( PadPosX[0] <= 106 )
		  PosX = PadPosX[0] + 7 + (PadPosX[0] - 106) / 5;
		if( PadPosX[0] > 106 && PadPosX[0] < 111 )
			PosX = PadPosX[0] + 7;
    if( PadPosX[0] >= 111 )
      PosX = PadPosX[0] + 7 + (PadPosX[0] - 111) / 5;
    PosY = static_cast<int>(PadPosY[0] - sqrt(256 - (PosX - PadPosX[0] - 7) *(PosX - PadPosX[0] - 7)) + 7);
    break;
  case 1:
		if( AnzPlayer == 2)
		{
      if( PadPosX[1] <= 106 )
  		  PosX = PadPosX[1] + 7 + (PadPosX[1] - 106) / 5;
  		if( PadPosX[1] > 106 && PadPosX[1] < 111 )
  			PosX = PadPosX[1] + 7;
      if( PadPosX[1] >= 111 )
				PosX = PadPosX[1] + 7 + (PadPosX[1] - 111) / 5;
      PosY = static_cast<int>(PadPosY[1] + sqrt(256 - (PosX - PadPosX[1] - 7) *(PosX - PadPosX[1] - 7)) + 7);

		}
		else
		{
    	Rotated = CalcRotPos( QPoint( PadPosX[1]+11 , PadPosY[1]+11 ) , 5 );

			Rotated.setX(Rotated.x()-11);
			Rotated.setY(Rotated.y()-11);

      if( Rotated.x() <= 106 )
  		  PosX = Rotated.x() + 7 + (Rotated.x() - 106) / 5;
  		if( Rotated.x() > 106 && Rotated.x() < 111 )
  			PosX = Rotated.x() + 7;
      if( Rotated.x() >= 111 )
				PosX = Rotated.x() + 7 + (Rotated.x() - 111) / 5;
      PosY = static_cast<int>(Rotated.y() + sqrt(256 - (PosX - Rotated.x() - 7) *(PosX - Rotated.x() - 7)) + 7);

			BackRot = CalcRotPos( QPoint( PosX+4 , PosY+4 ) , 1 );
      PosX = BackRot.x()-4;
			PosY = BackRot.y()-4;
		};
    break;
	case 2:
    Rotated = CalcRotPos( QPoint( PadPosX[2]+11 , PadPosY[2]+11 ) , 1 );

		Rotated.setX(Rotated.x()-11);
		Rotated.setY(Rotated.y()-11);

    if( Rotated.x() <= 106 )
		  PosX = Rotated.x() + 7 + (Rotated.x() - 106) / 5;
		if( Rotated.x() > 106 && Rotated.x() < 111 )
			PosX = Rotated.x() + 7;
    if( Rotated.x() >= 111 )
			PosX = Rotated.x() + 7 + (Rotated.x() - 111) / 5;
    PosY = static_cast<int>(Rotated.y() + sqrt(256 - (PosX - Rotated.x() - 7) *(PosX - Rotated.x() - 7)) + 7);

		BackRot = CalcRotPos( QPoint( PosX+4 , PosY+4 ) , 5 );
    PosX = BackRot.x()-4;
		PosY = BackRot.y()-4;

		break;
 };


 return QPoint( PosX , PosY );
}



void GameField::MoveSprites()
{
 //Bild neu erstellen
 static int PadRotate[3];
 int RotPos;

 bool Pad[3];
 bool Ball = false;

 int NewPadPosX[3];
 int NewPadPosY[3];

 int intBallPosX = static_cast<int>( BallPosX );
 int intBallPosY = static_cast<int>( BallPosY );
 int intBallPosX_old = static_cast<int>( BallPosX_old );
 int intBallPosY_old = static_cast<int>( BallPosY_old );

 int AnzStones;

 BallPosX_old = BallPosX;
 BallPosY_old = BallPosY;

 if( BallMode > 0)
 	Ball = true;

 for ( int i = 0 ; i < 3 ; i++ )
 {
 	Pad[i] = false;
  if( PadMode[i] > 0 )
 	{
  	Pad[i] = true;
  	PadRotate[i]++;
	  if( PadRotate[i] > 47 || PadRotate[i] < 0 )
	   PadRotate[i] = 0;
    NewPadPosX[i] = PadPosX[i];
    NewPadPosY[i] = PadPosY[i];
 	}
 };


 for( int i = 0 ; i < 3 ; i++ )
 {
  //Ausblenden des Pads
  // Einfgen des alten Hintergrunds an die alte Position des Sprites
  if( Pad[i])
   bitBlt( &gameimage, PadPosX_old[i], PadPosY_old[i], &padcopy, 24*i, 0, 24, 24, CopyROP, true );
 };


 //Ausblenden des Balles
 if( Ball )
  bitBlt( &gameimage, intBallPosX_old, intBallPosY_old, &ballcopy, 0, 0, 8, 8, CopyROP, true );

 if( RefreshStones ) //Mindestens ein Stein mu entfernt werden!
  {
   AnzStones = SetStones();
   RefreshStones = false;
   emit setpoints(PlayerPoints[0], PlayerPoints[1], PlayerPoints[2]);

	 if( AnzStones == 0 )
	   BallMode = LastPadTouch + 4;

  };


 if( GameMode == 0 )
 {
  //Game Over
  setBackgroundPixmap( gameimage );
  BallPosX_old = 1000;
  BallPosY_old = 1000;
  return;
 };


 //Ball zeichnen
 if( Ball )
 {
  // Kopieren des Hintergrunds, zu dem sich der Ball bewegt
  bitBlt( &ballcopy, 0, 0, &gameimage, intBallPosX, intBallPosY, 8, 8, CopyROP, true );
  // Ball zeichnen
  bitBlt( &gameimage, intBallPosX, intBallPosY, &ballimage, 0, 0, 8, 8, CopyROP , false );
 };

 //Pads in umgekehrter Reihenfolge zeichnen
 for( int i = 2 ; i >= 0 ; i-- )
 {
  if( Pad[i])
  {
   RotPos = PadRotate[i]/4;
   //2 Kopieren des Hintergrunds, zu dem sich der Pad bewegt
   bitBlt( &padcopy, 24*i, 0, &gameimage, NewPadPosX[i], NewPadPosY[i], 24, 24, CopyROP, true );
   // Pad zeichnen
   bitBlt( &gameimage, NewPadPosX[i], NewPadPosY[i], &padimage, 24*i, 24*RotPos, 24, 24, CopyROP , false );

   PadPosX_old[i]=NewPadPosX[i];
   PadPosY_old[i]=NewPadPosY[i];
  };
 };

 if( PadMode[0] == 1 || PadMode[0] == 2 )
  {
   //Tor 1zeichnen
   bitBlt( &gameimage, 80, 199, &tor1image, 0, 0, 80, 11, CopyROP, false );
  };

 if( PadMode[1] == 1 || PadMode[1] == 2 )
  {
  if( AnzPlayer == 3 )
  {
   // Tor 3 zeichnen
	SetLeftGoal( false );
//   bitBlt( &gameimage, 13, 20, &tor3image, 0, 0, 48, 72, CopyROP, false );
   // Ecke berschreiben
//   bitBlt( &gameimage, 0, 0, &gameimage, 0, 162, 32, 48, CopyROP, true );
//   bitBlt( &gameimage, 0, 48, &gameimage, 0, 162, 14, 28, CopyROP, true );
  };
  if( AnzPlayer == 2 )
  {
   // Tor 2 zeichnen
   bitBlt( &gameimage, 80, 0, &tor2image, 0, 0, 80, 11, CopyROP, false );
  };
 };

 if( PadMode[2] == 1 || PadMode[2] == 2 )
  {
   // Tor 4 zeichnen
   SetRightGoal( false );
// bitBlt( &gameimage, 179, 20, &tor4image, 0, 0, 48, 72, CopyROP, false );
   // Ecke berschreiben
//   bitBlt( &gameimage, 208, 0, &gameimage, 208, 162, 32, 48, CopyROP, true );
 //  bitBlt( &gameimage, 225, 48, &gameimage, 208, 162, 14, 28, CopyROP, true );
  };


 setBackgroundPixmap( gameimage );

}




