
/**********************************************************************
 *           gamefield.h  -  description 
 *           Project: "Hexatrolic"
 *           
 *           (C) 2002 by Helge Plehn
 *           29.01.2002
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by 
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                               
 *                                                                    
**********************************************************************/


#ifndef GAMEFIELD_H
#define GAMEFIELD_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qpaintdevice.h>
#include <qbitmap.h>
#include <qlcdnumber.h>
#include <qlabel.h>
//#include <qsound.h>

#include <qarray.h>
#include <stdio.h>


class GameField : public QWidget
{
Q_OBJECT
public:
 GameField( QWidget *parent=0, const char *name=0 );

public slots:
 void  SetPlayer( int Anzahl );

 void StartGame();
 void StopGame();
 void PauseGame();

 void PadMovement( QPoint Vektor );
 void spacepressed();

signals:
 //void angleChanged( int );
 //void forceChanged( int );
 void setpoints( int, int, int );
 void setgamemode( int );

protected:
 void timerEvent( QTimerEvent * );

 void mousePressEvent( QMouseEvent *);
 void mouseMoveEvent( QMouseEvent *e );

private:

	QLCDNumber *balls;
	QLCDNumber *level;
 	/*QLabel *lblballs;
   QLabel *lbllevel; */

 void DefineArrays();
 int SetStones();
 void MoveSprites();
 void SetPadPosition();
 void SetLeftGoal( bool SimpleCopy );
 void SetRightGoal( bool SimpleCopy );

 QPoint CalcPadPosition( QPoint MousePosition );
 QPoint CalcBallPosition( int PadNumber );
 QPoint CalcRotPos( QPoint Position , int Schritte );
 QPoint CalcCompPad( QPoint BallPos , QPoint OldPadPos );

 //Data-Arrays, used to be defined with: static const float SteinA[6];
 float SteinA[6];
 float SteinB[6][54];
 int SteinXpos[54];
 int SteinYpos[54];
 bool SteinOrientierung[54];
 int SteinFarbenLevel[12][54];
 int SteinFarbenAktuell[54];
 float SteinXmin[6][54];
 float SteinXmax[6][54];
 float SteinMinX[54];
 float SteinMaxX[54];
 float SteinMinY[54];
 float SteinMaxY[54];


 int GameMode;		/* 	0 = vor dem Start
 											1 = Pause
 											2 = Normal */

 int BallMode;		/*	0 = invisible
 											1 = at Pad1
 											2 = at Pad2
 											3 = at Pad3
 											4 = on the way to Pad1
 											5 = on the way to Pad2
 											6 = on the way to Pad3
 											7 = free move */

 int PadMode[3];	/*	0 = inisible
 											1 = move in
 											2 = move out
 											3 = steered by user
 											4 = steered by computer */     	

 int AnimationCounter[3];

 int GameLevel;
 int AnzBalls;
 int AnzPlayer;
 int PlayerPoints[3];
 int LastPadTouch;

 bool ServeBall;
 bool RefreshStones;

 float BallPosX;
 float BallPosY;
 float BallPosX_old;
 float BallPosY_old;
 float BallSpeedX;
 float BallSpeedY;
 float BalldX;
 float BalldY;

 QPoint PadMoveTo;
 QPoint PadSpeed;

 int PadPosX[3];
 int PadPosY[3];
 int PadPosX_old[3];
 int PadPosY_old[3];

 //Bitmaps
 QPixmap hexagonimage;
 QPixmap gameimage;
 QPixmap ballimage;
 QBitmap ballmask;
 QPixmap ballcopy;
 QPixmap padimage;
 QBitmap padmask;
 QPixmap padcopy;
 QPixmap stoneimage;
 QBitmap stonemask;
 QPixmap stonecopy;
 QPixmap tor1image;
 QBitmap tor1mask;
 QPixmap tor2image;
 QBitmap tor2mask;
 QPixmap tor3image;
 QBitmap tor3mask;
 QPixmap tor4image;
 QBitmap tor4mask;

};

#endif // GAMEFIELD_H


