/**********************************************************************
 *           hexatrolic.cpp  -  description
 *           Project: "Hexatrolic"
 *
 *           (C) 2002 by Helge Plehn
 *           29.11.2002
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
**********************************************************************/

#include "hexatrolic.h"
#include <qwidget.h>

#include <qfont.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpaintdevice.h>
#include <qstring.h>

#include <qlcdnumber.h>

#include <qpopupmenu.h>
#include <qmessagebox.h>

//#include <qpemenubar.h>

MyWidget::MyWidget( QWidget *parent, const char *name, WFlags fl )
: QWidget( parent, name, fl )
{
 setMinimumSize( 240, 286 );
 setMaximumSize( 240, 286 );

 setCaption( tr("Hexatrolic") );
 setBackgroundColor( QColor(0, 0, 0) );

 greenpoints = new QLCDNumber(10 , this, "greenpoints"  );
 greenpoints->setGeometry( 80, 25, 80, 15 );
 greenpoints->setFrameStyle ( 0 );
 greenpoints->setBackgroundColor( QColor(0, 0, 0) );
 greenpoints->setPalette ( QPalette ( QColor ( 0, 255, 0 ) ) );
 greenpoints->hide();

 redpoints = new QLCDNumber(10 , this, "redpoints"  );
 redpoints->setGeometry( 80, 42, 80, 15 );
 redpoints->setFrameStyle ( 0 );
 redpoints->setBackgroundColor( QColor(0, 0, 0) );
 redpoints->setPalette ( QPalette ( QColor ( 255, 0, 0 ) ) );
 redpoints->hide();

 bluepoints = new QLCDNumber(10 , this, "bluepoints"  );
 bluepoints->setGeometry(80, 59, 80, 15 );
 bluepoints->setBackgroundColor( QColor(0, 0, 0) );
 bluepoints->setFrameStyle ( 0 );
 bluepoints->setPalette ( QPalette ( QColor ( 0, 0, 255 ) ) );

 gameField = new GameField( this, "gameField" );
 gameField->setGeometry( 0, 76, 240, 210 );

 // define Menu

 QPopupMenu *game = new QPopupMenu();
 CHECK_PTR( game );
 game->insertItem( tr("&Start"), gameField, SLOT(StartGame()) );
 game->insertItem( tr("&Pause"), gameField, SLOT(PauseGame()) );
 game->insertItem( tr("Stop"), gameField, SLOT(StopGame()) );
 game->insertSeparator();
 //game->insertItem( "Highscores", this, SLOT(highscores()) );
 //game->insertSeparator();
 game->insertItem( tr("E&xit"),  this , SLOT(close()) );                   // eigentlich nicht "this",  sondern "qApp"

 isTwoPlayer = false;
 isThreePlayer = false;

 options = new QPopupMenu();
 CHECK_PTR( options );
 //options->insertItem( "Change Name", this, SLOT(changename()) );
 //options->insertSeparator();
 SingleID = options->insertItem( tr(" Single Player"), this, SLOT(singleplayer()) );
 TwoID = options->insertItem( tr(" Two Player"), this, SLOT(twoplayer()) );
 ThreeID = options->insertItem( tr(" Three Player"), this, SLOT(threeplayer()) );
 options->setCheckable( TRUE );
 options->setItemChecked( SingleID, true );

 QPopupMenu *help = new QPopupMenu;
 CHECK_PTR( help );
 help->insertItem( tr("&Help"), this, SLOT(help()));
 help->insertItem( tr("&About"), this, SLOT(about()) );

 menu = new QMenuBar( this );
 CHECK_PTR( menu );
 menu->insertItem( tr("&Game") , game );
 menu->insertItem( tr("&Options"), options );
 menu->insertSeparator();
 menu->insertItem( tr("&Help"), help );
 menu->setSeparator( QMenuBar::InWindowsStyle );

 connect( this ,SIGNAL(setplayer(int)), gameField ,SLOT(SetPlayer(int)));
 connect( this ,SIGNAL(SpacePressed()), gameField ,SLOT(spacepressed()));
 connect( this ,SIGNAL(PadMovement(QPoint)), gameField ,SLOT(PadMovement(QPoint)));

 connect( gameField ,SIGNAL(setpoints(int, int, int )), this ,SLOT(SetPoints(int, int, int)));
 connect( gameField ,SIGNAL(setgamemode(int)), this ,SLOT(setgamemode(int)));

 moveup = false;
 movedown = false;
 moveleft = false;
 moveright = false;
 lastleftright = 0;
 lastupdown = 0;

 gamemode = 0;
}


void MyWidget::about()
{
 QMessageBox::about( this, tr("About Hexatrolic"),
                         tr(" Hexatrolic 1.0 beta 3 \n\n"
                         " (c) 2002 by Helge Plehn\n"
                         " Helge_Plehn@web.de\n"
                         " This game is free \n software (GPL v2).\n")  );
}



void MyWidget::singleplayer()
{
 if( gamemode != 0 )
  return;

 isTwoPlayer = false;
 isThreePlayer = false;
 options->setItemChecked( SingleID, true );
 options->setItemChecked( TwoID, false );
 options->setItemChecked( ThreeID, false );

 greenpoints->hide();
 redpoints->hide();

 emit setplayer(1);
}

void MyWidget::twoplayer()
{
 if( gamemode != 0 )
  return;

 isTwoPlayer = true;
 isThreePlayer = false;
 options->setItemChecked( SingleID, false );
 options->setItemChecked( TwoID, true );
 options->setItemChecked( ThreeID, false );

 greenpoints->hide();
 redpoints->show();

 emit setplayer(2);
}

void MyWidget::threeplayer()
{
 if( gamemode != 0 )
  return;

 isTwoPlayer = false;
 isThreePlayer = true;
 options->setItemChecked( SingleID, false );
 options->setItemChecked( TwoID, false );
 options->setItemChecked( ThreeID, true );

 greenpoints->show();
 redpoints->show();

 emit setplayer(3);
}


void MyWidget::help()
{
 int answer;
  QMessageBox mb( tr("Hexatrolic") , tr(" Use pen or arrowkeys to move \n the blue ring. The ball is served \n by touching the screen or \n pressing the spacebar. \n Destroy the stones in the middle \n to gain single points. \n A goal scores 50 points in 3 Player \n games and 100 in 2 Player games. ")
							, QMessageBox::NoIcon, QMessageBox::Ok ,QMessageBox::NoButton , QMessageBox::NoButton , this );
  answer = mb.exec();
}

void MyWidget::SetPoints(int bpoints, int rpoints, int gpoints)
{
 bluepoints->display( bpoints );
 if (isTwoPlayer || isThreePlayer) redpoints->display( rpoints );
 if (isThreePlayer) greenpoints->display( gpoints );
}



void MyWidget::setgamemode(int Modus)
{
 static int highscore=0;
 int score;
 QString s1;
 QString s2;
 QString s3;
 QString s4;
 QString s5;
 QString s6;
 int answer;

 if( Modus == -1 )
 {
   //Game Over
  score = bluepoints->intValue();

  QString s6;
  if ( score <= highscore )
      s6 = tr("<qt>Game over!\n You've reached %1 points. Highscore is %2.</qt>").arg( score ).arg( highscore );

   // New Highscore
  if( score > highscore && highscore != 0) {
      s6 = tr("<qt>Game over!\n You've reached a new Highscore with %1 points</qt>").arg( score );
      highscore = score;
  }

  if( highscore == 0 ) {
      s6 = tr("<qt>Game over!\n. You've reached %1 points.<qt>").arg(highscore);
      highscore = score;
  }

  QMessageBox mb( tr("Hexatrolic") , s6 , QMessageBox::NoIcon, QMessageBox::Ok ,QMessageBox::NoButton , QMessageBox::NoButton , this );
  answer = mb.exec();
  Modus = 0;
          }
 gamemode = Modus;
}



  void MyWidget::keyPressEvent( QKeyEvent *k )
{
  int Taste;

  Taste = k->key();

	if( Taste == Key_Space ) emit SpacePressed();
	if( Taste == Key_Up ) {
		moveup = true;
		lastupdown = -1;
	};
	if( Taste == Key_Down ) {
		movedown = true;
		lastupdown = 1;
	};
	if( Taste == Key_Left ) {
		moveleft = true;
		lastleftright = -1;
  };
	if( Taste == Key_Right ) {
		moveright = true;
		lastleftright = 1;
	};

	SetPadMovement();
}


void MyWidget::keyReleaseEvent( QKeyEvent *k )
{
  int Taste;

  Taste = k->key();

	if( Taste == Key_Up ) moveup = false;
	if( Taste == Key_Down ) movedown = false;
	if( Taste == Key_Left ) moveleft = false;
	if( Taste == Key_Right ) moveright = false;

	SetPadMovement();
}


void MyWidget::SetPadMovement()
{
 	int vektorX = 0;
	int vektorY = 0;

  if( moveup && !movedown ) vektorY = - 3;
  if( movedown && !moveup ) vektorY = 3;
  if( movedown && moveup ) vektorY = 3 * lastupdown;
  if( moveleft && !moveright ) vektorX = - 5;
  if( moveright && !moveleft ) vektorX = 5;
  if( moveright && moveleft ) vektorX = 5 * lastleftright;

	emit PadMovement(QPoint(vektorX,vektorY));
}










